import 'dart:ui' as ui;

import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:phoenix_widgets/phoenix_webview.dart';

class PhoenixExpandableContentBlock extends StatefulWidget {
  const PhoenixExpandableContentBlock({
    Key? key,
    required this.text,
    required this.style,
    this.urls,
    required this.shortenButtonLabel,
    required this.expandButtonLabel,
    this.maxLines = 6,
    this.expandableStateChangeCallback,
    this.textCopiedCallback,
  }) : super(key: key);

  final String text;
  final TextStyle style;
  final List<Url>? urls;
  final String shortenButtonLabel;
  final String expandButtonLabel;
  final int maxLines;
  final VoidCallback? expandableStateChangeCallback;
  final VoidCallback? textCopiedCallback;

  @override
  State<PhoenixExpandableContentBlock> createState() =>
      _PhoenixExpandableContentBlockState();
}

class _PhoenixExpandableContentBlockState
    extends State<PhoenixExpandableContentBlock> {
  bool _isUnfold = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, size) {
        final span = TextSpan(text: widget.text, style: widget.style);
        final textPainter = TextPainter(
          text: span,
          maxLines: widget.maxLines,
          textDirection: ui.TextDirection.ltr,
        );
        textPainter.layout(maxWidth: size.maxWidth);
        if (textPainter.didExceedMaxLines) {
          return InkWell(
            onTap: _onTap,
            onLongPress: _onLongPress,
            child: SizedBox(
              width: MediaQuery.of(context).size.width,
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  _isUnfold
                      ? RichText(text: TextSpan(children: _textSpanLists()))
                      : RichText(
                          maxLines: widget.maxLines,
                          overflow: TextOverflow.ellipsis,
                          text: TextSpan(children: _textSpanLists()),
                        ),
                  GestureDetector(
                    behavior: HitTestBehavior.translucent,
                    onTap: _onTap,
                    child: Container(
                      padding: const EdgeInsets.only(top: 2.0),
                      child: Text(
                        _isUnfold
                            ? widget.shortenButtonLabel
                            : widget.expandButtonLabel,
                        style:
                            widget.style.apply(color: const Color(0xFF67C1BF)),
                      ),
                    ),
                  )
                ],
              ),
            ),
          );
        } else {
          return GestureDetector(
            onLongPress: _onLongPress,
            child: SizedBox(
              width: double.maxFinite,
              child: RichText(text: TextSpan(children: _textSpanLists())),
            ),
          );
        }
      },
    );
  }

  void _onTap() {
    setState(() {
      _isUnfold = !_isUnfold;
    });
    if (!_isUnfold) {
      widget.expandableStateChangeCallback?.call();
    }
  }

  void _onLongPress() {
    Clipboard.setData(ClipboardData(text: widget.text)).then((_) {
      widget.textCopiedCallback?.call();
    });
  }

  List<TextSpan> _textSpanLists() {
    if (widget.urls == null || widget.urls!.isEmpty) {
      return [TextSpan(text: widget.text, style: widget.style)];
    }

    List<TextSpan> urlTextSpanList = [
      TextSpan(
        text: widget.text.substring(0, widget.urls![0].indices![0]),
        style: widget.style,
      )
    ];

    for (int i = 0; i < widget.urls!.length; i++) {
      Url currentUrl = widget.urls![i];
      if (i != 0) {
        Url lastUrl = widget.urls![i - 1];
        urlTextSpanList.add(
          TextSpan(
            text: widget.text
                .substring(lastUrl.indices![1], currentUrl.indices![0]),
            style: widget.style,
          ),
        );
      }
      urlTextSpanList.add(
        TextSpan(
          text: widget.text
              .substring(currentUrl.indices![0], currentUrl.indices![1]),
          style: widget.style
              .apply(color: Colors.blue, decoration: TextDecoration.underline),
          recognizer: TapGestureRecognizer()
            ..onTap = () async {
              Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (context) =>
                      PhoenixWebview(url: currentUrl.url ?? "", title: ""),
                ),
              );
            },
        ),
      );
    }

    urlTextSpanList.add(
      TextSpan(
        text: widget.text.substring(widget.urls!.last.indices![1]),
        style: widget.style,
      ),
    );
    return urlTextSpanList;
  }
}

class Url {
  final String? url;
  final List<int>? indices;

  Url(this.url, this.indices);
}
